import datetime
import logging

from fw_file.dicom import DICOM, DICOMCollection
from fw_file.dicom.config import IMPLEMENTATION_CLASS_UID, IMPLEMENTATION_VERSION_NAME
from fw_file.dicom.utils import generate_uid
from pydicom.dataset import Dataset
from pydicom.sequence import Sequence
from tzlocal import get_localzone

from . import pkg_name

log = logging.getLogger(__name__)


def update_modified_dicom_info(dcm: DICOM) -> None:
    """Add OriginalAttributesSequence and Implementation information.

    Args:
        dcm (DICOM): DICOM to update.
    """
    log.debug(f"Populating OriginalAttributesSequence on {dcm.localpath}")
    time_zone = get_localzone()
    curr_dt = time_zone.localize(datetime.datetime.now())
    curr_dt_str = curr_dt.strftime("%Y%m%d%H%M%S.%f%z")
    add_implementation(dcm)
    # Modified attributes dataset
    mod_dat = Dataset()
    for element in dcm.tracker.data_elements:
        orig = element.export()["original"]
        mod_dat[orig.tag] = orig

    # Original attributes dataset
    orig_dat = Dataset()
    # Add Modified attributes dataset as a sequence
    orig_dat.ModifiedAttributesSequence = Sequence([mod_dat])

    orig_dat.ModifyingSystem = pkg_name

    orig_dat.ReasonForTheAttributeModification = "CORRECT"
    orig_dat.AttributeModificationDateTime = curr_dt_str

    raw = dcm.dataset.raw
    if not raw.get("OriginalAttributesSequence", None):
        raw.OriginalAttributesSequence = Sequence()
    raw.OriginalAttributesSequence.append(orig_dat)


def add_implementation(dcm: DICOM) -> None:
    """Sets implementation information to a dicom.

    Args:
        dcm (DICOM): DICOM to update.
    """
    log.debug(
        f"Adding ImplementationClassUID: {IMPLEMENTATION_CLASS_UID}, "
        f"ImplementationVersionName: {IMPLEMENTATION_VERSION_NAME}"
    )
    setattr(dcm, "ImplementationClassUID", IMPLEMENTATION_CLASS_UID)
    setattr(dcm, "ImplementationVersionName", IMPLEMENTATION_VERSION_NAME)


def add_missing_uid(dcms: DICOMCollection) -> bool:
    """Check for and add missing SeriesInstanceUID.

    Args:
        dcms (DICOMCollection): Dicom to check.

    Returns:
        (bool): Whether or not any modification was made.

    Raises:
        ValueError: When multiple SeriesInstanceUIDs are present across archive.
    """
    mod = False
    series_uid = None
    try:
        series_uid = dcms.get("SeriesInstanceUID")
    except ValueError as e:
        log.error("Multiple SeriesInstanceUIDs detected.  Please run splitter gear.")
        raise ValueError from e

    sops = dcms.bulk_get("SOPInstanceUID")
    if not all(sops):
        log.info("Populating missing SOPInstanceUIDs.")
        for dcm in dcms:
            if not dcm.get("SOPInstanceUID"):
                setattr(dcm, "SOPInstanceUID", generate_uid())
        mod = True

    if not series_uid:
        log.info("Populating missing SeriesInstanceUID.")
        series_uid = generate_uid()
        dcms.set("SeriesInstanceUID", series_uid)
        mod = True

    return mod
